var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { generateArrowLinks, sortByNamespace } from '../utils';
import { Page } from '../Page';
import { Api } from '../Api';
import { Dialog } from '../Dialog';
import { msg } from '../messenger';
import { TwinkleModule } from '../twinkleModule';
import { getPref } from '../Config';
/**
 * Module for batch page undeletions. Can undelete all non-existing pages linked
 * from the current page. Optionally their respective talk pages can also be
 * undeleted.
 *
 * Localisation: Should work without any configuration.
 */
var BatchUndeleteCore = /** @class */ (function (_super) {
    __extends(BatchUndeleteCore, _super);
    function BatchUndeleteCore() {
        var _this = _super.call(this) || this;
        _this.moduleName = 'batchundelete';
        _this.portletName = 'Und-batch';
        _this.portletId = 'twinkle-batchundelete';
        _this.portletTooltip = "Undelete 'em all";
        _this.windowTitle = 'Batch undelete';
        if (!Morebits.userIsSysop ||
            !mw.config.get('wgArticleId') ||
            (mw.config.get('wgNamespaceNumber') !== mw.config.get('wgNamespaceIds').user &&
                mw.config.get('wgNamespaceNumber') !== mw.config.get('wgNamespaceIds').project)) {
            return _this;
        }
        _this.addMenu();
        return _this;
    }
    BatchUndeleteCore.prototype.makeWindow = function () {
        var Window = new Dialog(600, 400);
        Window.setTitle(this.windowTitle);
        Window.setFooterLinks(this.footerlinks);
        var form = new Morebits.quickForm(this.evaluate.bind(this));
        form.append({
            type: 'checkbox',
            list: [
                {
                    label: msg('undbatch-restore-talks'),
                    name: 'undel_talk',
                    value: 'undel_talk',
                    checked: true,
                },
            ],
        });
        form.append({
            type: 'input',
            name: 'reason',
            label: 'Reason: ',
            size: 60,
        });
        var statusdiv = document.createElement('div');
        statusdiv.style.padding = '15px'; // just so it doesn't look broken
        Window.setContent(statusdiv);
        Morebits.status.init(statusdiv);
        Window.display();
        var query = {
            action: 'query',
            generator: 'links',
            prop: 'info',
            inprop: 'protection',
            titles: mw.config.get('wgPageName'),
            gpllimit: getPref('batchMax'),
            format: 'json',
        };
        var statelem = new Morebits.status(msg('fetching-list'));
        var wikipedia_api = new Api(msg('loading'), query);
        wikipedia_api.setStatusElement(statelem);
        wikipedia_api.post().then(function (apiobj) {
            var response = apiobj.getResponse();
            var pages = (response.query && response.query.pages) || [];
            pages = pages.filter(function (page) {
                return page.missing;
            });
            var list = [];
            pages.sort(sortByNamespace);
            pages.forEach(function (page) {
                var editProt = page.protection
                    .filter(function (pr) {
                    return pr.type === 'create' && pr.level === 'sysop';
                })
                    .pop();
                var title = page.title;
                list.push({
                    label: title +
                        (editProt
                            ? msg('word-separator') +
                                (editProt.expiry === 'infinity'
                                    ? msg('create-protected-indef')
                                    : msg('create-protected', new Morebits.date(editProt.expiry).calendar('utc')))
                            : ''),
                    value: title,
                    checked: true,
                    style: editProt ? 'color:red' : '',
                });
            });
            form.append({ type: 'header', label: msg('undbatch-pages-label') });
            form.append({
                type: 'button',
                label: msg('select-all'),
                event: function (e) {
                    $(Morebits.quickForm.getElements(e.target.form, 'pages')).prop('checked', true);
                },
            });
            form.append({
                type: 'button',
                label: msg('deselect-all'),
                event: function (e) {
                    $(Morebits.quickForm.getElements(e.target.form, 'pages')).prop('checked', false);
                },
            });
            form.append({
                type: 'checkbox',
                name: 'pages',
                shiftClickSupport: true,
                list: list,
            });
            form.append({ type: 'submit' });
            var result = form.render();
            Window.setContent(result);
            Morebits.quickForm.getElements(result, 'pages').forEach(generateArrowLinks);
        });
    };
    BatchUndeleteCore.prototype.evaluate = function (event) {
        var _this = this;
        var form = event.target;
        Morebits.wiki.actionCompleted.notice = msg('undbatch-complete');
        var numProtected = Morebits.quickForm.getElements(form, 'pages').filter(function (element) {
            return element.checked && element.nextElementSibling.style.color === 'red';
        }).length;
        if (numProtected > 0 && !confirm(msg('undbatch-confirm-protected', numProtected))) {
            return;
        }
        var input = Morebits.quickForm.getInputData(form);
        if (!input.reason) {
            return alert(msg('dbatch-give-reason'));
        }
        Morebits.simpleWindow.setButtonsEnabled(false);
        Morebits.status.init(form);
        if (!input.pages || !input.pages.length) {
            Morebits.status.error(msg('error'), msg('undbatch-no-pages'));
            return;
        }
        var pageUndeleter = new Morebits.batchOperation(msg('undeleting'));
        pageUndeleter.setOption('chunkSize', getPref('batchChunks'));
        pageUndeleter.setOption('preserveIndividualStatusLines', true);
        pageUndeleter.setPageList(input.pages);
        pageUndeleter.run(function (pageName) {
            var params = {
                page: pageName,
                undel_talk: input.undel_talk,
                reason: input.reason,
                pageUndeleter: pageUndeleter,
            };
            var wikipedia_page = new Page(pageName, msg('undeleting-page', pageName));
            wikipedia_page.setCallbackParameters(params);
            wikipedia_page.setEditSummary(input.reason);
            wikipedia_page.suppressProtectWarning();
            wikipedia_page.setMaxRetries(3); // temporary increase from 2 to make batchundelete more likely to succeed [[phab:T222402]] #613
            wikipedia_page.undeletePage().then(_this.doExtras.bind(_this), pageUndeleter.workerFailure);
        });
    };
    // this stupid parameter name is a temporary thing until I implement an overhaul
    // of Morebits.wiki.* callback parameters
    BatchUndeleteCore.prototype.doExtras = function (thingWithParameters) {
        var _this = this;
        var params = thingWithParameters.parent
            ? thingWithParameters.parent.getCallbackParameters()
            : thingWithParameters.getCallbackParameters();
        // the initial batch operation's job is to delete the page, and that has
        // succeeded by now
        params.pageUndeleter.workerSuccess(thingWithParameters);
        var query, wikipedia_api;
        if (params.undel_talk) {
            var talkpagename = new mw.Title(params.page).getTalkPage().getPrefixedText();
            if (talkpagename !== params.page) {
                query = {
                    action: 'query',
                    prop: 'deletedrevisions',
                    drvprop: 'ids',
                    drvlimit: 1,
                    titles: talkpagename,
                    format: 'json',
                };
                wikipedia_api = new Api(msg('check-talk-deleted'), query);
                wikipedia_api.params = params;
                wikipedia_api.params.talkPage = talkpagename;
                wikipedia_api.post().then(function () { return _this.undeleteTalk(wikipedia_api); });
            }
        }
    };
    BatchUndeleteCore.prototype.undeleteTalk = function (apiobj) {
        var page = apiobj.getResponse().query.pages[0];
        var exists = !page.missing;
        var delrevs = page.deletedrevisions && page.deletedrevisions[0].revid;
        if (exists || !delrevs) {
            // page exists or has no deleted revisions; forget about it
            return;
        }
        var talkpage = new Page(apiobj.params.talkPage, msg('undeleting-talk', apiobj.params.page));
        talkpage.setEditSummary(msg('undeleting-talk-summary', apiobj.params.page));
        talkpage.undeletePage();
    };
    BatchUndeleteCore.moduleName = 'batchundelete';
    return BatchUndeleteCore;
}(TwinkleModule));
export { BatchUndeleteCore };
//# sourceMappingURL=batchUndeleteCore.js.map