var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { TwinkleModule } from '../twinkleModule';
import { makeArray, obj_entries, obj_values, stripNs } from '../utils';
import { msg } from '../messenger';
import { Api } from '../Api';
import { Page } from '../Page';
import { Config, getPref } from '../Config';
import { Dialog } from '../Dialog';
/**
 * Module for tagging pages. For different types of pages, this module can be
 * configured to behave differently. The {@link TagMode} class should be extended
 * for each type of page (usually based on namespace).
 *
 * Define the following static field:
 * - {@link modeList} - which contains the array of classes you define that
 * extend {@link TagMode}
 *
 * Note that since this is a static field, it must be defined as TagCore.modeList
 * (not Tag.modeList).
 *
 * See enwiki localisation at https://github.com/wikimedia-gadgets/twinkle-enwiki/blob/master/src/tag.ts
 */
var TagCore = /** @class */ (function (_super) {
    __extends(TagCore, _super);
    function TagCore() {
        var _this = _super.call(this) || this;
        _this.moduleName = 'Tag';
        _this.portletName = 'Tag';
        _this.portletId = 'twinkle-tag';
        for (var _i = 0, _a = TagCore.modeList; _i < _a.length; _i++) {
            var mode = _a[_i];
            if (mode.isActive()) {
                // @ts-ignore
                _this.mode = new mode();
                break;
            }
        }
        if (!_this.mode) {
            return _this;
        }
        _this.portletTooltip = _this.mode.getMenuTooltip();
        _this.addMenu();
        return _this;
    }
    TagCore.prototype.makeWindow = function () {
        var Window = new Dialog(630, 500);
        // anyone got a good policy/guideline/info page/instructional page link??
        Window.setFooterLinks(this.footerlinks);
        this.mode.makeForm(Window);
        this.mode.formRender();
        this.mode.postRender();
    };
    TagCore.userPreferences = function () {
        return {
            title: 'Tag',
            preferences: [
                {
                    name: 'watchTaggedPages',
                    label: 'When tagging a page, how long to watch it for',
                    type: 'enum',
                    enumValues: Config.watchlistEnums,
                    default: 'no',
                },
                {
                    name: 'markTaggedPagesAsMinor',
                    label: 'Mark addition of tags as a minor edit',
                    type: 'boolean',
                    default: false,
                },
                {
                    name: 'markTaggedPagesAsPatrolled',
                    label: 'Check the "mark page as patrolled/reviewed" box by default',
                    type: 'boolean',
                    default: true,
                },
            ],
        };
    };
    /**
     * Adds a link to each template's description page
     * @param {Morebits.quickForm.element} checkbox  associated with the template
     */
    TagCore.makeArrowLinks = function (checkbox) {
        var link = Morebits.htmlNode('a', '>');
        link.setAttribute('class', 'tag-template-link');
        // @ts-ignore
        var tagname = checkbox.values;
        link.setAttribute('href', mw.util.getUrl((tagname.indexOf(':') === -1 ? 'Template:' : '') +
            (tagname.indexOf('|') === -1 ? tagname : tagname.slice(0, tagname.indexOf('|')))));
        link.setAttribute('target', '_blank');
        $(checkbox).parent().append('\u00A0', link);
    };
    /**
     * Generate an edit summary for the tagging and/or untagging
     * @param addedTags
     * @param removedTags
     * @param reason
     */
    TagCore.makeEditSummary = function (addedTags, removedTags, reason) {
        var makeTemplateLink = function (tag) {
            var text = '{{[[';
            // if it is a custom tag with a parameter
            if (tag.indexOf('|') !== -1) {
                tag = tag.slice(0, tag.indexOf('|'));
            }
            text += tag.indexOf(':') !== -1 ? tag : 'Template:' + tag + '|' + tag;
            return text + ']]}}';
        };
        var summaryText;
        if (addedTags.length && removedTags.length) {
            summaryText = msg('summary-added-removed', addedTags.map(makeTemplateLink), removedTags.map(makeTemplateLink), addedTags.length + removedTags.length);
        }
        else if (addedTags.length) {
            summaryText = msg('summary-added', addedTags.map(makeTemplateLink), addedTags.length);
        }
        else if (removedTags.length) {
            summaryText = msg('summary-removed', removedTags.map(makeTemplateLink), removedTags.length);
        }
        if (reason) {
            summaryText += msg('colon-separator') + reason;
        }
        // avoid long summaries
        if (summaryText.length > 499) {
            summaryText = summaryText.replace(/\[\[[^|]+\|([^\]]+)\]\]/g, '$1');
        }
        return summaryText;
    };
    TagCore.moduleName = 'Tag';
    return TagCore;
}(TwinkleModule));
export { TagCore };
/**
 * Abstract class for representing a type of page for the tag module.
 */
var TagMode = /** @class */ (function () {
    function TagMode() {
        this.existingTags = [];
        this.removalSupported = false; // Override to true for modes that support untagging
        /**
         * Name of grouping template.
         *
         * A group template is a template container that takes the form
         *
         * {{group template name|
         * {{tag template 1}}
         * {{tag template 2}}
         * }}
         *
         * Any other kind of "grouping" is not supported.
         *
         * If groupTemplateName is null, grouping is always disabled for the mode.
         *
         * Individual tags can be excluded from grouping by setting excludeInGroup=true
         * for that template in tag data configuration.
         *
         * Grouping can also be disabled depending on user input or other factors by
         * setting this.params.disableGrouping = true in the preprocessParams() hook.
         *
         * Default: null, which indicates no grouping template for the mode.
         */
        this.groupTemplateName = null;
        /**
         * Minimum number of tags that a group should contain
         */
        this.groupMinSize = 1;
        /**
         * Should tags that are not present in tag configuration be considered
         * groupable? This includes custom tags set in user-level preferences,
         * and tags found existing on the page which are absent in the config.
         */
        this.assumeUnknownTagsGroupable = true;
    }
    TagMode.isActive = function () {
        // must be overridden
        return false;
    };
    /**
     * Removal of tags is possible only if this returns true:
     * when the user is viewing the article in read mode or a permalink
     * of the latest version.
     */
    TagMode.prototype.canRemove = function () {
        return (this.removalSupported &&
            // Only on latest version of pages
            mw.config.get('wgCurRevisionId') === mw.config.get('wgRevisionId') &&
            // Disabled on latest diff because the diff slider could be used to slide
            // away from the latest diff without causing the script to reload
            !mw.config.get('wgDiffNewId'));
    };
    /**
     * Returns the text used as the tooltip for the portlet
     */
    TagMode.prototype.getMenuTooltip = function () {
        return 'Add maintenance tags to the page';
    };
    /**
     * Returns the string used as the Twinkle dialog's title
     */
    TagMode.prototype.getWindowTitle = function () {
        return 'Add maintenance tags';
    };
    TagMode.prototype.makeForm = function (Window) {
        var _this = this;
        this.Window = Window;
        this.Window.setTitle(this.getWindowTitle());
        this.form = new Morebits.quickForm(function () { return _this.evaluate(); });
        this.constructFlatObject();
        this.form.append({
            type: 'input',
            label: msg('search-tags'),
            name: 'quickfilter',
            size: '30px',
            event: QuickFilter.onInputChange,
        });
        if (this.removalSupported && !this.canRemove()) {
            this.form.append({
                type: 'div',
                name: 'untagnotice',
                label: Morebits.htmlNode('div', msg('untag-from-read')),
            });
        }
        this.scrollbox = this.form.append({
            type: 'div',
            id: 'tagWorkArea',
            className: 'morebits-scrollbox',
            style: 'max-height: 28em',
        });
        this.parseExistingTags();
        this.makeExistingTagList(this.scrollbox);
        this.makeTagList(this.scrollbox);
        var customTags = getPref(this.getCustomTagPrefName());
        if (customTags && customTags.length) {
            this.scrollbox.append({ type: 'header', label: 'Custom tags' });
            this.scrollbox.append({ type: 'checkbox', name: 'tags', list: customTags });
        }
    };
    TagMode.prototype.getCustomTagPrefName = function () {
        return 'custom' + Morebits.string.toUpperCaseFirstChar(this.name) + 'TagList';
    };
    /**
     * Generates the tag list in the GUI.
     * @param container
     */
    TagMode.prototype.makeTagList = function (container) {
        var _this = this;
        if (Array.isArray(this.tagList)) {
            this.makeTagListGroup(this.tagList, container);
        }
        else {
            $.each(this.tagList, function (groupName, group) {
                container.append({ type: 'header', label: groupName });
                if (Array.isArray(group)) {
                    // if group is a list of tags
                    _this.makeTagListGroup(group, container);
                }
                else {
                    // if group is a list of subgroups
                    var subdiv_1 = container.append({ type: 'div' });
                    $.each(group, function (subgroupName, subgroup) {
                        subdiv_1.append({ type: 'div', label: [Morebits.htmlNode('b', subgroupName)] });
                        _this.makeTagListGroup(subgroup, subdiv_1);
                    });
                }
            });
        }
    };
    /**
     * Helper function for {@link makeTagList}
     * @param list
     * @param container
     */
    TagMode.prototype.makeTagListGroup = function (list, container) {
        var _this = this;
        var excludeTags = new Set(this.existingTags.filter(function (t) { var _a; return !((_a = _this.flatObject[t]) === null || _a === void 0 ? void 0 : _a.dupeAllowed); }));
        container.append({
            type: 'checkbox',
            name: 'tags',
            list: list
                .filter(function (item) { return !excludeTags.has(item.tag); })
                .map(function (item) { return ({
                label: '{{' + item.tag + '}}' + (item.description ? ': ' + item.description : ''),
                value: item.tag,
                subgroup: item.subgroup,
            }); }),
        });
    };
    /**
     * Make the HTML for the existing tags.
     * @param container
     */
    TagMode.prototype.makeExistingTagList = function (container) {
        var _this = this;
        if (!this.existingTags.length) {
            return;
        }
        container.append({ type: 'header', label: msg('tags-present-header') });
        var tagConfigs = this.existingTags.map(function (tag) {
            return _this.flatObject[tag] || { tag: tag };
        });
        container.append({
            type: 'checkbox',
            name: 'existingTags',
            list: tagConfigs.map(function (item) { return ({
                label: '{{' + item.tag + '}}' + (item.description ? ': ' + item.description : ''),
                value: item.tag,
                checked: true,
                style: 'font-style: italic',
            }); }),
        });
    };
    /**
     * Parse existing tags. This is NOT asynchronous.
     * Should be overridden for tag modes where removalSupported is true.
     * Populate this.existingTags with the names of tags present on the page.
     */
    TagMode.prototype.parseExistingTags = function () { };
    /**
     * Create a flat object for speeding up lookup for the tag properties
     */
    TagMode.prototype.constructFlatObject = function () {
        var _this = this;
        this.flatObject = {};
        if (Array.isArray(this.tagList)) {
            // this.tagList is of type tagData[]
            this.tagList.forEach(function (item) {
                _this.flatObject[item.tag] = item;
            });
        }
        else {
            obj_values(this.tagList).forEach(function (group) {
                //  what's wrong with this type?
                obj_values(group).forEach(function (subgroup) {
                    if (Array.isArray(subgroup)) {
                        subgroup.forEach(function (item) {
                            _this.flatObject[item.tag] = item;
                        });
                    }
                    else {
                        _this.flatObject[subgroup.tag] = subgroup;
                    }
                });
            });
        }
    };
    TagMode.prototype.formAppendPatrolLink = function () {
        if (!document.getElementsByClassName('patrollink').length) {
            return;
        }
        this.form.append({
            type: 'checkbox',
            list: [
                {
                    label: msg('mark-patrolled'),
                    value: 'patrol',
                    name: 'patrol',
                    checked: getPref('markTaggedPagesAsPatrolled'),
                },
            ],
        });
    };
    TagMode.prototype.formRender = function () {
        this.form.append({
            type: 'submit',
            className: 'tw-tag-submit',
        });
        this.result = this.form.render();
        this.Window.setContent(this.result);
        this.Window.display();
    };
    /**
     * Actions carried out after the form has been rendered and the Dialog has become visible.
     */
    TagMode.prototype.postRender = function () {
        QuickFilter.init(this.result);
        Morebits.quickForm.getElements(this.result, 'tags').forEach(TagCore.makeArrowLinks);
        Morebits.quickForm.getElements(this.result, 'existingTags').forEach(TagCore.makeArrowLinks);
        // style adjustments
        $(this.scrollbox).find('h5').css({ 'font-size': '110%' });
        $(this.scrollbox).find('h5:not(:first-child)').css({ 'margin-top': '1em' });
        $(this.scrollbox).find('div').filter(':has(span.quickformDescription)').css({ 'margin-top': '0.4em' });
        // Add status text node after Submit button
        var $status = $('<small>').attr('id', 'tw-tag-status');
        $status.insertAfter($('button.tw-tag-submit'));
        var addedCount = 0, removedCount = 0;
        // tally tags added/removed, update statusNode text
        $('[name=tags], [name=existingTags]').on('click', function (e) {
            var checkbox = e.target;
            if (checkbox.name === 'tags') {
                addedCount += checkbox.checked ? 1 : -1;
            }
            else if (checkbox.name === 'existingTags') {
                removedCount += checkbox.checked ? -1 : 1;
            }
            var statusText = '';
            if (addedCount && removedCount) {
                statusText = msg('status-added-removed', addedCount, removedCount);
            }
            else if (addedCount) {
                statusText = msg('status-added', addedCount);
            }
            else if (removedCount) {
                statusText = msg('status-removed', removedCount);
            }
            $status.text('  ' + statusText);
        });
    };
    /**
     * Invoked when the form is submitted.
     */
    TagMode.prototype.evaluate = function () {
        var _this = this;
        this.captureFormData();
        var validationMessage = this.checkInputs();
        if (validationMessage) {
            return alert(validationMessage);
        }
        this.preprocessParams();
        Morebits.simpleWindow.setButtonsEnabled(false);
        Morebits.status.init(this.result);
        this.action().then(function () {
            Morebits.status.actionCompleted(msg('tag-complete', _this.name));
            setTimeout(function () {
                window.location.href = mw.util.getUrl(Morebits.pageNameNorm, { redirect: 'no' });
            }, 1e9);
        });
    };
    /**
     * Gather the data the user filled into the form.
     */
    TagMode.prototype.captureFormData = function () {
        this.params = Morebits.quickForm.getInputData(this.result);
        this.params.tagsToRemove = this.result.getUnchecked('existingTags'); // XXX: Morebits-defined function
        this.params.tagsToRetain = this.params.existingTags || [];
    };
    /**
     * Validate input. Return a string in case of issues; this string is used as the message for a browser prompt().
     * Return nothing if all inputs are valid. Use {@link validateInput} for customisation.
     * @internal
     * @sealed
     */
    TagMode.prototype.checkInputs = function () {
        // Check if any tag is selected or if any already present tag is deselected
        if (this.params.tags.length === 0 && (!this.canRemove() || this.params.tagsToRemove.length === 0)) {
            return msg('select-one');
        }
        return this.validateInput();
    };
    /**
     * If inputs are invalid, return a string that is shown to the user via alert().
     * If inputs are valid, don't return anything.
     */
    TagMode.prototype.validateInput = function () { };
    TagMode.prototype.preprocessParams = function () {
        this.getTemplateParameters();
    };
    TagMode.prototype.getTemplateParameters = function () {
        var _this = this;
        this.templateParams = {};
        this.params.tags.forEach(function (tag) {
            _this.templateParams[tag] = {};
            var subgroupObj = _this.flatObject[tag] && _this.flatObject[tag].subgroup;
            makeArray(subgroupObj).forEach(function (gr) {
                if (gr.parameter && (_this.params[gr.name] || gr.required)) {
                    _this.templateParams[tag][gr.parameter] = _this.params[gr.name] || '';
                }
            });
        });
    };
    /**
     * Get the regex to be used to search for or remove a tag.
     * @param tag
     */
    TagMode.prototype.getTagRegex = function (tag) {
        return new RegExp('\\{\\{' + Morebits.pageNameRegex(tag) + '\\s*(\\|[^}]*)?\\}\\}\\n?');
    };
    /**
     * This function assumes that grouping is enabled in the first place.
     * @param tag
     */
    TagMode.prototype.isGroupable = function (tag) {
        return this.flatObject[tag] ? !this.flatObject[tag].excludeInGroup : this.assumeUnknownTagsGroupable;
    };
    /**
     * Get regex that matches the start of the grouping template (the opening braces plus
     * the template name. The template name is captured as a regex group (pun unintended).
     */
    TagMode.prototype.groupRegex = function () {
        var regexString = '\\{\\{\\s*(' + this.groupTemplateNameRegex + ')\\s*(?:\\||\\}\\})';
        return new RegExp(regexString, this.groupTemplateNameRegexFlags);
    };
    /**
     * Generate the parameter text from the data stored in this.templateParams.
     * this.templateParams is populated automatically based on param names from the configs
     * and values from user input. If any additional changes to templateParams are required,
     * you can do that in preprocessParams().
     * @param tag
     */
    TagMode.prototype.getParameterText = function (tag) {
        if (!this.templateParams[tag]) {
            mw.log.warn('this.templateParams[tag] undefined');
            return '';
        }
        return obj_entries(this.templateParams[tag])
            .map(function (_a) {
            var key = _a[0], value = _a[1];
            return "|" + key + "=" + value;
        })
            .join('');
    };
    TagMode.prototype.getTagText = function (tag) {
        var subst = this.flatObject[tag] && this.flatObject[tag].subst ? 'subst:' : '';
        return '{{' + subst + tag + this.getParameterText(tag) + '}}';
    };
    TagMode.prototype.makeTagSetText = function (tags) {
        var _this = this;
        return tags.map(function (tag) { return _this.getTagText(tag) + '\n'; }).join('');
    };
    TagMode.prototype.addTagsOutsideGroup = function (tags) {
        var tagText = this.makeTagSetText(tags);
        this.pageText = this.insertTagText(tagText, this.pageText);
    };
    /**
     * If the tag is present in pageText, removes it from pageText and adds it to
     * params.groupableExistingTagsText.
     * @param tag
     */
    TagMode.prototype.shiftTag = function (tag) {
        var _this = this;
        var isShifted = false; // Avoid a .test() before the .replace() causing 2 regex searches
        this.pageText = this.pageText.replace(this.getTagRegex(tag), function (tagText) {
            isShifted = true;
            _this.params.groupableExistingTagsText += tagText.trim() + '\n'; // add to groupableExistingTagsText
            return ''; // remove from pageText
        });
        return isShifted;
    };
    /**
     * Get the wikitext of the groupable existing tags, removes it from the
     * page text and adds them to params.groupableExistingTagsText, which
     * the returned promise resolves to.
     */
    TagMode.prototype.spliceGroupableExistingTags = function () {
        var _this = this;
        this.params.groupableExistingTagsText = '';
        var tagsToShiftAsync = this.params.groupableExistingTags.filter(function (tag) {
            return !_this.shiftTag(tag);
        });
        if (tagsToShiftAsync.length === 0) {
            return $.Deferred().resolve(this.params.groupableExistingTagsText);
        }
        var api = new Api(msg('getting-redirects'), this.getRedirectsQuery(tagsToShiftAsync));
        return api.post().then(function (apiobj) {
            var pages = apiobj.getResponse().query.pages.filter(function (p) {
                return !p.missing && !!p.linkshere;
            });
            pages.forEach(function (page) {
                var shifted = _this.shiftTag(stripNs(page.title));
                if (!shifted) {
                    shifted = page.linkshere.some(function (template) {
                        var tag = stripNs(template.title);
                        return _this.shiftTag(tag);
                    });
                }
                if (!shifted) {
                    // unnecessary message?
                    new Morebits.status('Note', msg('cant-reposition', stripNs(page.title)));
                }
            });
            return _this.params.groupableExistingTagsText;
        });
    };
    /**
     * Remove tag from {@link pageText}, if it exists.
     * @param tag
     * @returns true if tag was removed, false otherwise
     */
    TagMode.prototype.removeTemplate = function (tag) {
        var isRemoved = false; // Avoid a .test() before the .replace() causing 2 regex searches
        this.pageText = this.pageText.replace(this.getTagRegex(tag), function () {
            isRemoved = true;
            return '';
        });
        return isRemoved;
    };
    /**
     * Get the API query used for querying redirects to the template
     * @param tags
     */
    TagMode.prototype.getRedirectsQuery = function (tags) {
        return {
            action: 'query',
            prop: 'linkshere',
            titles: tags.map(function (pg) { return 'Template:' + pg; }),
            redirects: 1,
            lhnamespace: '10',
            lhshow: 'redirect',
            lhlimit: 'max',
            format: 'json',
        };
    };
    /**
     * Remove tags from {@link pageText}
     */
    TagMode.prototype.removeTags = function () {
        var _this = this;
        var params = this.params;
        if (!params.tagsToRemove.length) {
            return $.Deferred().resolve();
        }
        Morebits.status.info(msg('untagging'), msg('removing'));
        var tagsToRemoveAsync = params.tagsToRemove.filter(function (tag) {
            return !_this.removeTemplate(tag);
        });
        if (tagsToRemoveAsync.length === 0) {
            return $.Deferred().resolve();
        }
        // Remove tags which appear in page text as redirects
        var api = new Api(msg('tag-fetching-redirects', tagsToRemoveAsync.map(function (t) { return '{{' + t + '}}'; })), this.getRedirectsQuery(tagsToRemoveAsync));
        return api.post().then(function (apiobj) {
            var pages = apiobj.getResponse().query.pages.filter(function (p) {
                return (!p.missing && !!p.linkshere) || Morebits.status.warn(msg('info'), msg('cant-remove', stripNs(p.title)));
            });
            (apiobj.getResponse().query.redirects || []).forEach(function (_a) {
                var from = _a.from, to = _a.to;
                new Morebits.status('Note', msg('resolved-redirect', stripNs(from), stripNs(to)));
            });
            pages.forEach(function (page) {
                var removed = _this.removeTemplate(stripNs(page.title));
                if (!removed) {
                    removed = page.linkshere.some(function (template) {
                        var tag = stripNs(template.title);
                        return _this.removeTemplate(tag);
                    });
                }
                if (!removed) {
                    Morebits.status.warn(msg('note'), msg('cant-remove', stripNs(page.title)));
                }
            });
        });
    };
    /**
     * Any initial cleanup of the page.
     */
    TagMode.prototype.initialCleanup = function () { };
    /**
     * Returns true if a group template is to be added to the page, otherwise false.
     */
    TagMode.prototype.shouldAddGroup = function () {
        var params = this.params;
        return (this.groupTemplateName &&
            !params.disableGrouping &&
            params.groupableExistingTags.length + params.groupableNewTags.length >= this.groupMinSize);
    };
    /**
     * Given that the group exists on `pageText` (either added by us now or existed before),
     * move given `tagText` into the group
     * @param tagText
     */
    TagMode.prototype.addTagsIntoGroup = function (tagText) {
        if (!tagText) {
            if (tagText === undefined)
                throw new Error('tagText undefined');
            return;
        }
        var groupRgxExec = this.groupRegex().exec(this.pageText);
        // Add new tags into group, and put the updated group wikitext into this.pageText
        var miRegex = new RegExp('(\\{\\{\\s*' + // Opening braces
            groupRgxExec[1] + // template name
            // XXX: unnecessarily overspecific from this point onwards
            '\\s*(?:\\|(?:\\{\\{[^{}]*\\}\\}|[^{}])*)?)' + // ??? Copied from friendlytag
            '\\}\\}\\s*', // Closing braces, followed by spaces/newlines
        'im');
        this.pageText = this.pageText.replace(miRegex, '$1' + tagText + '}}\n');
    };
    /**
     * Controls how the final text of new tags is inserted into the page.
     * Defaults to just putting them at the very top of the page, along with two newlines.
     * You may want to override this if you want them to go below any hatnotes or deletion
     * notices (use Morebits.wikitext.page#insertAfterTemplates, see enwiki ArticleMode).
     * @param tagText
     * @param pageText
     */
    TagMode.prototype.insertTagText = function (tagText, pageText) {
        return tagText + '\n' + pageText;
    };
    /**
     * Any final cleanup of the page.
     */
    TagMode.prototype.finalCleanup = function () {
        var _this = this;
        if (!this.groupTemplateName || this.params.groupingDisabled) {
            return;
        }
        // Remove any groups containing less than minGroupSize tags
        // XXX: This might misbehave if existing tags in the MI have parameters
        // that contain nested templates.
        // TODO: use regex-less parsing.
        var nItemGroupRegex = function (n) {
            var start = '\\{\\{\\s*' + _this.groupTemplateNameRegex + '\\s*\\|\\s*(';
            var tags = '(?:\\{\\{[^}]+\\}\\}\\s*){' + n + '}';
            var end = ')\\}\\}\\n?';
            var regexString = start + tags + end;
            return new RegExp(regexString, _this.groupTemplateNameRegexFlags);
        };
        // unbind substs of time parser functions
        var unbinder = new Morebits.unbinder(this.pageText);
        unbinder.unbind('\\{\\{subst:CURRENT', '\\}\\}');
        for (var i = 0; i < this.groupMinSize; i++) {
            unbinder.content = unbinder.content.replace(nItemGroupRegex(i), '$1');
        }
        this.pageText = unbinder.rebind();
    };
    TagMode.prototype.action = function () {
        var _this = this;
        this.pageobj = new Page(Morebits.pageNameNorm, msg('tagging-status', this.name));
        return this.pageobj.load().then(function () {
            _this.pageText = _this.pageobj.getPageText();
            _this.initialCleanup();
            _this.sortTags();
            return $.when(_this.addAndRearrangeTags(), _this.removeTags()).then(function () {
                _this.finalCleanup();
                return _this.savePage();
            });
        });
    };
    TagMode.prototype.sortTags = function () { };
    TagMode.prototype.addAndRearrangeTags = function () {
        this.pageText = this.insertTagText(this.makeTagSetText(this.params.tags), this.pageText);
    };
    TagMode.prototype.savePage = function () {
        this.pageobj.setPageText(this.pageText);
        this.pageobj.setEditSummary(TagCore.makeEditSummary(this.params.tags, this.params.tagsToRemove, this.params.reason));
        this.pageobj.setWatchlist(getPref('watchTaggedPages'));
        this.pageobj.setMinorEdit(getPref('markTaggedPagesAsMinor'));
        this.pageobj.setCreateOption('nocreate');
        if (this.params.patrol) {
            this.pageobj.triage();
        }
        return this.pageobj.save();
    };
    return TagMode;
}());
export { TagMode };
var QuickFilter = /** @class */ (function () {
    function QuickFilter() {
    }
    QuickFilter.init = function (result) {
        QuickFilter.$allCheckboxDivs = $(result).find('[name=tags], [name=existingTags]').parent();
        QuickFilter.$allHeaders = $(result).find('h5, .quickformDescription');
        result.quickfilter.focus(); // place cursor in the quick filter field as soon as window is opened
        result.quickfilter.autocomplete = 'off'; // disable browser suggestions
        result.quickfilter.addEventListener('keypress', function (e) {
            if (e.keyCode === 13) {
                // prevent enter key from accidentally submitting the form
                e.preventDefault();
                return false;
            }
        });
    };
    QuickFilter.onInputChange = function () {
        // flush the DOM of all existing underline spans
        QuickFilter.$allCheckboxDivs.find('.search-hit').each(function (i, e) {
            var label_element = e.parentElement;
            // This would convert <label>Hello <span class=search-hit>wo</span>rld</label>
            // to <label>Hello world</label>
            label_element.innerHTML = label_element.textContent;
        });
        if (this.value) {
            QuickFilter.$allCheckboxDivs.hide();
            QuickFilter.$allHeaders.hide();
            var searchString = this.value;
            var searchRegex = new RegExp(mw.util.escapeRegExp(searchString), 'i');
            QuickFilter.$allCheckboxDivs.find('label').each(function () {
                var label_text = this.textContent;
                var searchHit = searchRegex.exec(label_text);
                if (searchHit) {
                    var range = document.createRange();
                    var textnode = this.childNodes[0];
                    range.selectNodeContents(textnode);
                    range.setStart(textnode, searchHit.index);
                    range.setEnd(textnode, searchHit.index + searchString.length);
                    var underline_span = $('<span>').addClass('search-hit').css('text-decoration', 'underline')[0];
                    range.surroundContents(underline_span);
                    this.parentElement.style.display = 'block'; // show
                }
            });
        }
        else {
            QuickFilter.$allCheckboxDivs.show();
            QuickFilter.$allHeaders.show();
        }
    };
    return QuickFilter;
}());
export { QuickFilter };
//# sourceMappingURL=tagCore.js.map