var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { Config, setDefaultConfig } from './Config';
import { addPortletLink } from './portlet';
import { userDisabledModules } from './init';
import { Twinkle } from './twinkle';
/**
 * Base class for all Twinkle modules
 */
var TwinkleModule = /** @class */ (function () {
    function TwinkleModule() {
    }
    /**
     * Return a PreferenceGroup with the preferences added by this module
     */
    TwinkleModule.userPreferences = function () { };
    /**
     * Add a preference to this module's preference group
     * @param pref
     */
    TwinkleModule.prototype.addPreference = function (pref) {
        Config.addPreference(this.moduleName, pref);
    };
    /**
     * Executes just before the menu is added, suitable place for tweaking
     * portlet configurations.
     */
    TwinkleModule.prototype.beforeAddMenu = function () { };
    /**
     * Add menu portlet
     */
    TwinkleModule.prototype.addMenu = function () {
        var _this = this;
        this.beforeAddMenu();
        addPortletLink(function () { return _this.makeWindow(); }, this.portletName, this.portletId || 'twinkle-' + this.moduleName.toLowerCase(), this.portletTooltip);
    };
    /**
     * Generate the GUI dialog for this module. Invoked when the portlet
     * generated by addMenu is clicked.
     */
    TwinkleModule.prototype.makeWindow = function () { };
    return TwinkleModule;
}());
export { TwinkleModule };
export function registerModule(module) {
    var prefs = module.userPreferences();
    if (prefs) {
        Config.addGroup(module.moduleName, __assign(__assign({}, prefs), { module: module.moduleName }));
        setDefaultConfig(prefs.preferences.map(function (pref) {
            return {
                name: pref.name,
                value: pref.default,
            };
        }));
    }
    if (!userDisabledModules.includes(module.moduleName)) {
        // new module() inits the module, which usually means adding the portlet link
        // the result is assigned to Twinkle object only for debugging purposes
        Twinkle[module.moduleName.toLowerCase()] = new module();
    }
}
//# sourceMappingURL=twinkleModule.js.map