var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { SiteConfig } from './siteConfig';
var ApiError = /** @class */ (function (_super) {
    __extends(ApiError, _super);
    function ApiError() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ApiError;
}(Error));
export { ApiError };
export function makeArray(obj) {
    if (!obj) {
        return [];
    }
    if (Array.isArray(obj)) {
        return obj;
    }
    return [obj];
}
/**
 * Check if page is a redirect given the page text.
 * @param text
 */
export function isTextRedirect(text) {
    return SiteConfig.redirectTagAliases
        .map(function (str) {
        return new RegExp('^\\s*' + str, 'i');
    })
        .some(function (regex) {
        return regex.test(text);
    });
}
/**
 * Remove namespace name from title if present
 * Exception-safe wrapper around mw.Title
 * @param {string} title
 */
export function stripNs(title) {
    var title_obj = mw.Title.newFromUserInput(title);
    if (!title_obj) {
        return title; // user entered invalid input; do nothing
    }
    return title_obj.getNameText();
}
/**
 * Add namespace name to page title if not already given
 * CAUTION: namespace name won't be added if a namespace (*not* necessarily
 * the same as the one given) already is there in the title
 * @param {string} title
 * @param {number} namespaceNumber
 */
export function addNs(title, namespaceNumber) {
    var title_obj = mw.Title.newFromUserInput(title, namespaceNumber);
    if (!title_obj) {
        return title; // user entered invalid input; do nothing
    }
    return title_obj.toText();
}
/**
 * Get URL parameter.
 * Alias for mw.util.getParamValue
 * @param param
 */
export function urlParamValue(param) {
    return mw.util.getParamValue(param);
}
export function link(displaytext, title, params) {
    return "<a target=\"_blank\" href=\"" + mw.util.getUrl(title, params) + "\">" + displaytext + "</a>";
}
/**
 * Used in batch, unlink, and deprod to sort pages by namespace, as
 * json formatversion=2 sorts by pageid instead (#1251)
 */
export function sortByNamespace(first, second) {
    return first.ns - second.ns || (first.title > second.title ? 1 : -1);
}
/**
 * Used in batch listings to link to the page in question with >
 */
export function generateArrowLinks(checkbox) {
    var link = Morebits.htmlNode('a', ' >');
    link.setAttribute('class', 'tw-arrowpage-link');
    link.setAttribute('href', mw.util.getUrl(checkbox.value));
    link.setAttribute('target', '_blank');
    checkbox.nextElementSibling.append(link);
}
/**
 * Used in deprod and unlink listings to link the page title
 */
export function generateBatchPageLinks(checkbox) {
    var $checkbox = $(checkbox);
    var link = Morebits.htmlNode('a', $checkbox.val());
    link.setAttribute('class', 'tw-batchpage-link');
    link.setAttribute('href', mw.util.getUrl($checkbox.val()));
    link.setAttribute('target', '_blank');
    $checkbox.next().prepend([link, ' ']);
}
export function makeOptoutLink(module) {
    if (!module) {
        return '';
    }
    return 'notwinkle.test/?module=' + module;
}
/**
 * Make template wikitext from the template name and parameters
 * @param {string} name - name of the template. Include "subst:" if necessary
 * @param {Object} parameters - object with keys and values being the template param names and values.
 * Use numbers as keys for unnamed parameters.
 * If a value is falsy (undefined or null or empty string), the param doesn't appear in output.
 * @returns {string}
 */
export function makeTemplate(name, parameters) {
    var parameterText = obj_entries(parameters)
        .filter(function (_a) {
        var k = _a[0], v = _a[1];
        return !!v;
    }) // ignore params with no value
        .map(function (_a) {
        var name = _a[0], value = _a[1];
        return "|" + name + "=" + value;
    })
        .join('');
    return '{{' + name + parameterText + '}}';
}
// Non-polluting shims for common ES6 functions
/**
 * Shim for Object.values
 * @param obj
 */
export function obj_values(obj) {
    // @ts-ignore
    return Object.values ? Object.values(obj) : Object.keys(obj).map(function (k) { return obj[k]; });
}
/**
 * Shim for Object.entries
 * @param obj
 */
export function obj_entries(obj) {
    // @ts-ignore
    return Object.entries ? Object.entries(obj) : Object.keys(obj).map(function (k) { return [k, obj[k]]; });
}
/**
 * Shim for Object.fromEntries
 * @param entries
 */
export function obj_fromEntries(entries) {
    // @ts-ignore
    if (Object.fromEntries) {
        // @ts-ignore
        return Object.fromEntries(entries);
    }
    var obj = {};
    for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
        var _a = entries_1[_i], key = _a[0], val = _a[1];
        obj[key] = val;
    }
    return obj;
}
/**
 * Shim for Array.prototype.includes
 * @param arr
 * @param item
 * @deprecated as this is now shimmed in MediaWiki
 */
export function arr_includes(arr, item) {
    return arr.indexOf(item) !== -1;
}
/**
 * Shim for Array.prototype.find
 * @param arr
 * @param predicate
 * @deprecated as this is now shimmed in MediaWiki
 */
export function arr_find(arr, predicate) {
    return Array.prototype.find ? arr.find(predicate) : arr.filter(predicate)[0];
}
/**
 * Shim for Array.prototype.flatMap
 * @param arr
 * @param callbackfn
 */
export function arr_flatMap(arr, callbackfn) {
    // @ts-ignore
    if (Array.prototype.flatMap) {
        // @ts-ignore
        return arr.flatMap(callbackfn);
    }
    return arr.map(callbackfn).reduce(function (previousValue, currentValue) { return previousValue.concat(currentValue); }, []);
}
/**
 * Shim for String.prototype.includes
 * @param str
 * @param item
 */
export function str_includes(str, item) {
    return str.indexOf(item) !== -1;
}
/**
 * Shim for String.prototype.startsWith
 * @param str
 * @param text
 */
export function str_startsWith(str, text) {
    // @ts-ignore
    return String.prototype.startsWith ? str.startsWith(text) : str.indexOf(text) === 0;
}
/**
 * Shim for String.prototype.endsWith
 * @param str
 * @param text
 */
export function str_endsWith(str, text) {
    // @ts-ignore
    if (String.prototype.endsWith) {
        // @ts-ignore
        return str.endsWith(text);
    }
    else {
        var lastIdx = str.lastIndexOf(text);
        return lastIdx !== -1 && lastIdx === str.length - text.length;
    }
}
//# sourceMappingURL=utils.js.map